/* button-test.c -- Test of buttons
   Copyright 2000 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of EBCS.

EBCS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

EBCS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with EBCS; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/param.h>
#include <sys/locks.h>
#include <gel/timer.h>
#include <sys/sio.h>
#include <gel/buttons.h>
#include <ebcs/remote.h>
#include <ebcs/board.h>
#include <benchs.h>
#include <stdio.h>
#include <test.h>
#include <lcd.h>

void print_keys (void);
void send_cmd (unsigned char c);
void button_event_handler (event_callout *callout, event *ev);

#if 0
void
timer_benchs ()
{
  bench_t b;
  struct timeval tv;
  struct timeval d2;

  bench_init (&b);

  bench_start (&b);
  timer_gettime (&tv);
  bench_stop (&b);
  bench_report (&b, "timer_gettime");

  bench_start (&b);
  timer_gettime (&d2);
  bench_stop (&b);
  bench_report (&b, "timer_gettime (2)");

  bench_start (&b);
  time_add (&d2, &tv);
  bench_stop (&b);
  bench_report (&b, "time_add");

  bench_start (&b);
  time_sub (&d2, &tv);
  bench_stop (&b);
  bench_report (&b, "time_sub");
}
#endif
#if 1
#define EVENT_QUEUE_SIZE 16
static event event_queue[EVENT_QUEUE_SIZE];

static unsigned short prev_report;

void
print_keys ()
{
  unsigned short mask = button_get_pressed ();
  unsigned short delta;

  lock ();
  delta = mask ^ prev_report;
  prev_report = mask;
  unlock ();

  if (delta)
    printf ("[%lx]", (long) mask);
  
  if (delta & 1)
    printf ("%s", mask & 1 ? "+1" : "-1");
  if (delta & 2)
    printf ("%s", mask & 2 ? "+2" : "-2");
  if (delta & 4)
    printf ("%s", mask & 4 ? "+3" : "-3");
  if (delta & 8)
    printf ("%s", mask & 8 ? "+8" : "-8");
}
#endif
void
send_cmd (unsigned char c)
{
  c = c ^ 0x0ff;
  remote_putc (c);
}

event_callout test_button_callout;

void
button_event_handler (event_callout *callout, event *ev)
{
  print_keys ();
}

unsigned char stop = 0;

int
main()
{
  serial_init ();
  remote_initialize ();
#if 0
  while (1)
    remote_putc (~(0x80 | LCD_BIT_4 | LCD_BIT_5 | LCD_BIT_6 | LCD_BIT_7));
#else
  timer_initialize ();
  event_initialize (event_queue, EVENT_QUEUE_SIZE);
  button_initialize ();
  button_enable ();

  test_depend ("serial:timer:event");
  test_start ("button");

  event_register (&button_change_event, &test_button_callout,
                  button_event_handler, 0);

  event_loop (1000 * 31 * 65535, &stop);
  test_end ("button");
#if 0  
  while (1)
    {
      event_wait (100000, &button_change_event);
      print_keys ();
    }
#endif
#endif
  return 0;
}
