/* lcd-test.c -- Test of timer and LCD
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of EBCS.

EBCS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

EBCS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with EBCS; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/*! @page lcd-test Test program for the LCD operations.

  This program tests the LCD operation by displaying patterns on
  the LCD panel.

  @htmlonly
  Source file: <a href="lcd-test_c-source.html">lcd-test.c</a>
  @endhtmlonly

 */

#include <sys/param.h>
#include <sys/locks.h>
#include <sys/sio.h>
#include <lcd.h>
#include <sys/spi.h>
#include <time.h>
#include <test.h>
#include <stdio.h>
#include <benchs.h>

void
lcd_benchs ()
{
  bench_t b;

  bench_init (&b);

  bench_start (&b);
  lcd_clear ();
  bench_stop (&b);
  bench_report (&b, "lcd_reset");

  bench_start (&b);
  lcd_print_at (0, 0, "Hello World");
  bench_stop (&b);
  bench_report (&b, "lcd_print_at %ld chars", (long) strlen ("Hello World"));

  lcd_blink (0);
  bench_start (&b);
  lcd_blink (0);
  bench_stop (&b);
  bench_report (&b, "lcd_blink same state");

  bench_start (&b);
  lcd_blink (1);
  bench_stop (&b);
  bench_report (&b, "lcd_blink new state");
  
  bench_start (&b);
  lcd_goto (0, 0);
  bench_stop (&b);
  bench_report (&b, "lcd_goto");

  bench_start (&b);
  lcd_clear ();
  bench_stop (&b);
  bench_report (&b, "lcd_clear");

}

int
main ()
{
  unsigned long i;
  int result;
  int j;

  serial_init ();
  test_depend ("serial");
  test_start ("lcd-test");

  lock ();
  /* Reset the LCD controller.  */
  lcd_reset ();

  test_repeat = 4;
  for (j = 0; j < test_repeat; j++) {
    printf ("Doing %ld\n", (long) j);

    lcd_print_at (0, 0, "Hello World!    ");
    lcd_print_at (0, 1, "Lcd test running");
    udelay (2000000);

    lcd_blink (1);
    for (i = 0; i < 15; i++)
      {
        lcd_goto (i, 0);
        udelay (1000000);
      }
    lcd_print_at (0, 0, "Lcd test running");
    lcd_print_at (0, 1, "Hello World!    ");
    udelay (2000000);

    lcd_blink (0);
    for (i = 0; i < 15; i++)
      {
        lcd_goto (i, 0);
        udelay (1000000);
      }

    lcd_clear ();
    lcd_print ("Cleared");

    udelay (2000000);
  }
  result = test_end ("lcd-test");
  if (result == 0)
    {
      lcd_benchs ();
    }
  return result;
}
